#
# Copyright (c) 2008-2018 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'

from fanstatic import ConfigurationError

from pyams_content.component.illustration import IIllustration
from pyams_content.component.paragraph.interfaces.html import IHTMLParagraph, IRawParagraph
from pyams_content.features.renderer.interfaces import IContentRenderer
from pyams_default_theme import library
from pyams_default_theme.features.renderer import BaseContentRenderer
from pyams_skin.layer import IPyAMSLayer
from pyams_template.template import template_config
from pyams_utils.adapter import adapter_config, get_annotation_adapter
from pyams_utils.fanstatic import ExternalResource
from pyams_utils.pygments import IPygmentsCodeConfiguration, PygmentsCodeRendererSettings, render_source

from pyams_default_theme import _


#
# Raw paragraph default renderer
#

@adapter_config(name='default', context=(IRawParagraph, IPyAMSLayer), provides=IContentRenderer)
@template_config(template='templates/raw-default.pt', layer=IPyAMSLayer)
class RawParagraphDefaultRenderer(BaseContentRenderer):
    """Raw paragraph default renderer"""

    label = _("Default raw HTML renderer")
    weight = 1

    i18n_context_attrs = ('title', 'body')


#
# Raw paragraph source code renderer
#

RAW_PARAGRAPH_SOURCE_CODE_RENDERER_SETTINGS_KEY = 'pyams_content.raw.renderer:source-code'


@adapter_config(context=IRawParagraph, provides=IPygmentsCodeConfiguration)
def raw_paragraph_source_code_renderer_settings_factory(context):
    """Raw paragraph source code renderer settings factory"""
    return get_annotation_adapter(context, RAW_PARAGRAPH_SOURCE_CODE_RENDERER_SETTINGS_KEY,
                                  PygmentsCodeRendererSettings)


@adapter_config(name='source-code', context=(IRawParagraph, IPyAMSLayer), provides=IContentRenderer)
@template_config(template='templates/raw-source-code.pt', layer=IPyAMSLayer)
class RawParagraphSourceCodeRenderer(BaseContentRenderer):
    """Raw paragraph source code renderer"""

    label = _("Formatted source code renderer")
    weight = 10

    settings_interface = IPygmentsCodeConfiguration

    i18n_context_attrs = ('title', 'body')

    @property
    def resources(self):
        path = 'get-pygments-style.css?style={}'.format(self.settings.style)
        try:
            yield ExternalResource(library, path, resource_type='css')
        except ConfigurationError:
            yield library.known_resources[path]

    @property
    def source(self):
        code = self.body
        if code:
            return render_source(code, self.settings)
        return ''


#
# HTML paragraph default renderer
#

@adapter_config(name='default', context=(IHTMLParagraph, IPyAMSLayer), provides=IContentRenderer)
@template_config(template='templates/html-default.pt', layer=IPyAMSLayer)
class HTMLParagraphDefaultRenderer(BaseContentRenderer):
    """HTML paragraph default renderer"""

    label = _("Default rich text renderer")

    i18n_context_attrs = ('title', 'body',)

    illustration = None
    illustration_renderer = None

    def update(self):
        super(HTMLParagraphDefaultRenderer, self).update()
        self.illustration = IIllustration(self.context)
        if self.illustration.data:
            renderer = self.illustration_renderer = self.illustration.get_renderer(self.request)
            if renderer is not None:
                renderer.update()
